import sys

from paiagym.config import ENV
from paiagym.utils import import_game, import_script

def training() -> None:
    # env variables
    game_name = ENV.get('GAME_NAME', '')
    input_scripts = ENV.get('INPUT_SCRIPTS', 'ml_play.py').split(':')

    Game = import_game(game_name)
    game = Game()
    env = game.make_env()

    MLPlay = import_script(input_scripts[0])
    player = MLPlay(env)
    
    player.learn(total_timesteps=100_000)
    env.close()


if __name__ == '__main__':
    game_name = None
    input_scripts = None
    if len(sys.argv) > 2:
        game_name = sys.argv[1]
        input_scripts = sys.argv[2]
    
    if game_name is not None:
        ENV['GAME_NAME'] = game_name
    if input_scripts is not None:
        ENV['INPUT_SCRIPTS'] = input_scripts
    
    training()


import json
import os
from gymnasium import Wrapper

from paiagym.config import ENV, bool_ENV, int_ENV

class GameData:
    def __init__(self):
        self.action = None
        self.observation = None
        self.reward = None
        self.terminated = None
        self.truncated = None
        self.info = None

class PAIAWrapper(Wrapper):
    def __init__(self, env, on_start=None, on_step=None, on_finish=None):
        super().__init__(env)
        self.env = env
        self.game_data = GameData()
        self.result = None
        self._on_start = self.on_start if on_start is None else on_start
        self._on_step = self.on_step if on_step is None else on_step
        self._on_finish = self.on_finish if on_finish is None else on_finish
    
    def reset(self, *, seed=None, options=None):
        observation, info = self.env.reset(seed=seed, options=options)
        self.game_data.observation = observation
        self.game_data.info = info

        self._on_start(self.env, self.game_data)
        self.result = self._on_step(self.env, self.game_data)
        
        return observation, info

    def step(self, action):
        observation, reward, terminated, truncated, info = self.env.step(action)
        self.game_data.action = action
        self.game_data.observation = observation
        self.game_data.reward = reward
        self.game_data.terminated = terminated
        self.game_data.truncated = truncated
        self.game_data.info = info
        
        self.result = self._on_step(self.env, self.game_data)

        if terminated or truncated:
            self._on_finish(self.env, self.game_data, self.result)

        return observation, reward, terminated, truncated, info
    
    def on_start(self, env, game_data):
        # environment variables
        record_video = bool_ENV('RECORD_VIDEO', False)
        width = int_ENV('VIDEO_WIDTH', 1920)
        height = int_ENV('VIDEO_HEIGHT', 1080)
        fullscreen = bool_ENV('VIDEO_FULLSCREEN', False)
        
        if record_video:
            env.unwrapped.begin_render(screen_width=width, screen_height=height, fullscreen=fullscreen)
    
    def on_step(self, env, game_data):
        return None # result
    
    def on_finish(self, env, game_data, result):
        # environment variables
        save_game_result = bool_ENV('SAVE_GAME_RESULT', False)
        game_result_dir = os.path.abspath(ENV.get('GAME_RESULT_DIR', 'result'))
        game_result_filename = ENV.get('GAME_RESULT_FILENAME', 'game_result.json')
        record_video = bool_ENV('RECORD_VIDEO', False)
        video_filename = ENV.get('VIDEO_FILENAME', 'video.mp4')

        if record_video or save_game_result:
            if not os.path.exists(game_result_dir):
                os.makedirs(game_result_dir)

        if record_video:
            env.unwrapped.end_render()
            video = env.render()
            video_path = os.path.join(game_result_dir, video_filename)
            with open(video_path, 'wb') as fout:
                fout.write(video)
                print(f'Video saved at {video_path}')
        
        game_result_path = os.path.join(game_result_dir, game_result_filename)
        with open(game_result_path, 'w') as fout:
            game_result = [
                {
                    'player': '1P',
                    'progress': result['progress'],
                    'used_time': result['used_time']
                }
            ]
            print(game_result)
            if save_game_result:
                json.dump(game_result, fout, indent=4)
                print(f'Game result saved at {game_result_path}')


import os
from sys import platform

class PAIAGame:
    def __init__(self):
        self.name = None # for unity_path
    
    def make_env(self):
        pass

    def unity_path(self, game_dir: str):
        '''
        override with the following in the new Game:

        return super().unity_path(os.path.dirname(__file__))
        '''
        binary_path = os.path.join(game_dir, 'build')
        game_path = None
        if platform == 'win32':
            game_path = os.path.join(binary_path, 'windows', f'{self.name}', f'{self.name}.exe')
        elif platform == 'linux' or platform == 'linux2':
            game_path = os.path.join(binary_path, 'linux', f'{self.name}.x86_64')
        elif platform == 'darwin':
            game_path = os.path.join(binary_path, 'mac', f'{self.name}.app')
        if os.path.exists(game_path):
            return game_path
        else:
            return None
    
    def on_start(self, env, game_data):
        pass
        
    def on_step(self, env, game_data):
        pass

    def on_finish(self, env, game_data, result):
        pass


import os
import numpy as np
import gymnasium as gym
from gymnasium import Wrapper, spaces
import gymize

from paiagym import PAIAGame, PAIAWrapper, GameData
from paiagym.config import ENV, bool_ENV

def kart_env(file_name: str=None, render_mode: str=None):
    observation_space = spaces.Dict(
        {
            # The RaySensor is composed of the distance to the hit object and hit object
			# The item that Raycast hit, record it will a specific item number.
			# RayHit
			# 0: No hit
			# 1: Wall
			# 2: Car
			# 3: Gas
			# 4: Wheel
			# 5: Nitro
			# 6: Turtle
			# 7: Banana
            'RaySensor': spaces.Dict(
                {
                    'Front': spaces.Dict(
                        {
                            'Distance': spaces.Box(low=0, high=np.inf, shape=(1,), dtype=np.float32),
                            'Hit': spaces.Discrete(8)
                        }
                    ),
                    'FrontRight': spaces.Dict(
                        {
                            'Distance': spaces.Box(low=0, high=np.inf, shape=(1,), dtype=np.float32),
                            'Hit': spaces.Discrete(8)
                        }
                    ),
                    'FrontLeft': spaces.Dict(
                        {
                            'Distance': spaces.Box(low=0, high=np.inf, shape=(1,), dtype=np.float32),
                            'Hit': spaces.Discrete(8)
                        }
                    ),
                    'Left': spaces.Dict(
                        {
                            'Distance': spaces.Box(low=0, high=np.inf, shape=(1,), dtype=np.float32),
                            'Hit': spaces.Discrete(8)
                        }
                    ),
                    'Right': spaces.Dict(
                        {
                            'Distance': spaces.Box(low=0, high=np.inf, shape=(1,), dtype=np.float32),
                            'Hit': spaces.Discrete(8)
                        }
                    ),
                    'Back': spaces.Dict(
                        {
                            'Distance': spaces.Box(low=0, high=np.inf, shape=(1,), dtype=np.float32),
                            'Hit': spaces.Discrete(8)
                        }
                    ),
                    'BackRight': spaces.Dict(
                        {
                            'Distance': spaces.Box(low=0, high=np.inf, shape=(1,), dtype=np.float32),
                            'Hit': spaces.Discrete(8)
                        }
                    ),
                    'BackLeft': spaces.Dict(
                        {
                            'Distance': spaces.Box(low=0, high=np.inf, shape=(1,), dtype=np.float32),
                            'Hit': spaces.Discrete(8)
                        }
                    )
                }
                
            ),
            # The image that camera captures, include front and back:
            'CameraFront': spaces.Box(low=0, high=255, shape=(112, 252, 3), dtype=np.uint8),
            'CameraBack': spaces.Box(low=0, high=255, shape=(112, 252, 3), dtype=np.uint8),
            # A floating point number that shows the current progress:
			'Progress': spaces.Box(low=-1, high=1, shape=(1,), dtype=np.float32),
			# A floating point number that shows how much time has been used:
			'UsedTime': spaces.Box(low=0, high=np.inf, shape=(1,), dtype=np.float32),
			# A floating point number that shows current velocity:
			'Velocity': spaces.Box(low=0, high=np.inf, shape=(1,), dtype=np.float32),
			# The amount of refill remains, refills = (gas, wheels):
			'RefillRemaining': spaces.Box(low=0, high=1, shape=(2,), dtype=np.float32),
			# The number of effect remains, effects = (nitro, banana, turtle):
			'EffectRemaining': spaces.Box(low=0, high=np.iinfo(np.int32).max, shape=(3,), dtype=np.int32)
        }
    )

    action_space = spaces.Dict(
        {
            'Acceleration': spaces.Box(low=0, high=1, dtype=np.bool_),
            'Brake': spaces.Box(low=0, high=1, dtype=np.bool_),
            'Steering': spaces.Box(low=-1, high=1, dtype=np.float32),
        }
    )

    env = gym.make('gymize/Unity-v0', env_name='kart3d', file_name=file_name, observation_space=observation_space, action_space=action_space, render_mode=render_mode)
    
    return env


class KartWrapper(Wrapper):
    def __init__(self, env):
        super().__init__(env)

        self.observation_space = spaces.Dict(
            {
                # Rays: Front 0, FrontRight 1, FrontLeft 2, Left 3, Right 4, Back 5, BackRight 6, BackLeft 7
                'RaySensor': spaces.Box(low=0, high=np.inf, shape=(8,), dtype=np.float32),
                # The image that camera captures, include front and back:
                'CameraFront': spaces.Box(low=0, high=255, shape=(112, 252, 3), dtype=np.uint8),
                'CameraBack': spaces.Box(low=0, high=255, shape=(112, 252, 3), dtype=np.uint8),
                # A floating point number that shows the current progress:
                'Progress': spaces.Box(low=-1, high=1, shape=(1,), dtype=np.float32),
                # A floating point number that shows how much time has been used:
                'UsedTime': spaces.Box(low=0, high=np.inf, shape=(1,), dtype=np.float32),
                # A floating point number that shows current velocity:
                'Velocity': spaces.Box(low=0, high=np.inf, shape=(1,), dtype=np.float32),
                # The amount of refill remains, refills = (gas, wheels):
                'RefillRemaining': spaces.Box(low=0, high=1, shape=(2,), dtype=np.float32),
                # The number of effect remains, effects = (nitro, banana, turtle):
                'EffectRemaining': spaces.Box(low=0, high=np.iinfo(np.int32).max, shape=(3,), dtype=np.int32)
            }
        )
        self.action_space = spaces.Discrete(3) # 0: Left, 1: Foward, 2: Right

        self.action_mapping = [
            { 'Acceleration': False, 'Brake': False, 'Steering': -1.0 },
            { 'Acceleration': True, 'Brake': False, 'Steering': 0.0 },
            { 'Acceleration': False, 'Brake': False, 'Steering': 1.0 }
        ]
    
    def reset(self, *, seed=None, options=None):
        observation, info = self.env.reset(seed=seed, options=options)
        obs = self._obs(observation)
        return obs, info
    
    def step(self, action):
        observation, reward, terminated, truncated, info = self.env.step(self.action_mapping[action])
        obs = self._obs(observation)
        return obs, reward, terminated, truncated, info
    
    def _obs(self, observation):
        obs = { key: value for key, value in observation.items() }
        ray_sensor = [
            observation['RaySensor']['Front']['Distance'][0],
            observation['RaySensor']['FrontRight']['Distance'][0],
            observation['RaySensor']['FrontLeft']['Distance'][0],
            observation['RaySensor']['Left']['Distance'][0],
            observation['RaySensor']['Right']['Distance'][0],
            observation['RaySensor']['Back']['Distance'][0],
            observation['RaySensor']['BackRight']['Distance'][0],
            observation['RaySensor']['BackLeft']['Distance'][0]
        ]
        obs['RaySensor'] = np.array(ray_sensor, dtype=np.float32)
        return obs


class KartInfoWrapper(Wrapper):
    def reset(self, *, seed=None, options=None):
        observation, info = self.env.reset(seed=seed, options=options)

        random_seed = ''
        if not bool_ENV('IS_PICKUP', False):
            random_seed = None
        else:
            random_seed = ENV.get('PICKUP_MAP', '')
        self.env.unwrapped.send_info(agent='', info=random_seed) # info: seed in string format, if is None: not using PickUps, if is '': using PickUps with auto random seed, if is string number, then its the fixed seed

        kart_name = ENV.get('1P_NAME', 'KART')
        self.env.unwrapped.send_info(agent='agent', info=kart_name) # info: kart name

        return observation, info


class Game(PAIAGame):
    def __init__(self, *args, **kwargs):
        self.name = 'kart3d' # for unity_path

    def make_env(self):
        env = kart_env(file_name=self.unity_path(), render_mode='video')
        env = KartWrapper(env)
        env = KartInfoWrapper(env)
        env = PAIAWrapper(env, on_step=self.on_step)
        return env
    
    def unity_path(self):
        return super().unity_path(os.path.dirname(__file__))
    
    def on_step(self, env, game_data: GameData):
        # return the game result
        if game_data.observation is not None:
            return {
                'progress': float(game_data.observation['Progress']),
                'used_time': float(game_data.observation['UsedTime'])
            }
        else:
            return None


import argparse

parser = argparse.ArgumentParser()


parser.add_argument('-v', '--verbosity', help='increase output verbosity')
parser.add_argument('-i', '--input-ai', action='append', help='the scripts of players')

args = parser.parse_args()

print(args.input_ai)


import ast
import logging
import os

from dotenv import load_dotenv

def recursive_load(path='.env', required_envs=[]):
    os.environ.pop('REQUIRE', None)
    path = os.path.abspath(path)
    load_dotenv(path)
    if 'REQUIRE' not in os.environ:
        return
    try:
        include_paths = ast.literal_eval(os.environ['REQUIRE'])
    except:
        include_paths = os.environ['REQUIRE']
    if isinstance(include_paths, str):
        if os.path.abspath(include_paths) not in required_envs:
            recursive_load(include_paths, required_envs + [path])
    for include_path in include_paths:
        if os.path.abspath(include_path) not in required_envs:
            recursive_load(include_path, required_envs + [path])

recursive_load()
ENV = os.environ # You can get and set the environment variable by ENV['VAR_NAME']
'''
Remember to import in every possible files:
from config import ENV

Just as common Dict:
You have to check if the key exists first: if 'KEY' in ENV
Access empty key will cause KeyError,
Or you can use ENV.get()
Or ENV.setdefault(key[,default]), this will set the value also
You can only insert the string value

ENV[key] = value can set the value

You can use REQUIRE key to include .env files (in string or list format)
'''

def to_bool(value, default=None):
    if isinstance(value, str):
        if value == '0': return False
        elif value.upper() == 'FALSE': return False
        elif value == '1': return True
        elif value.upper() == 'TRUE': return True
    return default

def bool_ENV(key, default: bool=None):
    return to_bool(ENV.get(key), default)

def to_int(value, default: int=None):
    if default is None:
        return None
    else:
        try:
            return int(value or 0)
        except ValueError:
            return None

def int_ENV(key, default: int=None):
    return to_int(ENV.get(key), default)

def to_float(value, default: int=None):
    if default is None:
        return None
    else:
        try:
            return float(value or 0.0)
        except ValueError:
            return None

def float_ENV(key, default: float=None):
    return to_float(ENV.get(key), default)

# Logging Level
if ENV.get('LOG_LEVEL') == 'CRITICAL': log_level = logging.CRITICAL
elif ENV.get('LOG_LEVEL') == 'ERROR': log_level = logging.ERROR
elif ENV.get('LOG_LEVEL') == 'WARNING': log_level = logging.WARNING
elif ENV.get('LOG_LEVEL') == 'DEBUG': log_level = logging.DEBUG
elif ENV.get('LOG_LEVEL') == 'NOTSET': log_level = logging.NOTSET
else: log_level = logging.INFO
logging.basicConfig(level=log_level, format='%(message)s')


from gymnasium import Wrapper
from paiagym.config import bool_ENV

class KartWrapper(Wrapper):
    def __init__(self, env):
        super().__init__(env)

        self.early_stop = bool_ENV('EARLY_STOP', False)

        self.progress = 0
        self.cnt = 0
    
    def reset(self, *, seed=None, options=None):
        self.progress = 0
        self.cnt = 0
        observation, info = self.env.reset(seed=seed, options=options)
        return observation, info

    def step(self, action):
        observation, reward, terminated, truncated, info = self.env.step(action)
        reward = self.new_reward(observation['Progress'], observation['RaySensor'])
        if reward <= -0.1:
            self.cnt += 1
        if self.cnt > 100:
            if self.early_stop:
                truncated = True
        return observation, reward, terminated, truncated, info
    
    def new_reward(self, progress, rays):
        reward = 0

        if rays[0] < 0.5:
            return -0.1

        if progress > self.progress:
            reward = 0.1 + float(progress * 10)
            self.progress = progress
        elif progress < self.progress:
            reward = -0.1 + float((progress - self.progress) * 1000)
        else:
            reward = -0.01
        
        if rays[0] > 0.5 and rays[1] > 0.5 and rays[2] > 0.5 and rays[3] > 0.5 and rays[4] > 0.5:
            reward += 0.01
        
        return reward


import sys
from paiagym.config import ENV
from paiagym.manage import add, install, list_all_games, list_available_games, remove, uninstall, update
from paiagym.inferencing import inferencing
from paiagym.training import training

def main():
    games = []
    game = None
    path = None
    input_scripts = None

    if len(sys.argv) > 1:
        op = sys.argv[1]
        if op == 'ls':
            games = []
            if len(sys.argv) > 2:
                if sys.argv[2] == '-a':
                    games = list_all_games()
                if sys.argv[2] == '-m':
                    if len(sys.argv) > 3:
                        mode = sys.argv[3]
                        games = list_available_games(mode)
            else:
                games = list_available_games()
            print('\n'.join(games))
        elif op == 'add':
            if len(sys.argv) > 2:
                game = sys.argv[2]
                if len(sys.argv) > 3:
                    path = sys.argv[3]
                add(game, path)
        elif op == 'install':
            if len(sys.argv) > 2:
                game = sys.argv[2]
                install(game)
        elif op == 'update':
            if len(sys.argv) > 2:
                game = sys.argv[2]
                if len(sys.argv) > 3:
                    path = sys.argv[3]
                update(game, path)
        elif op == 'remove':
            if len(sys.argv) > 2:
                game = sys.argv[2]
                remove(game)
        elif op == 'uninstall':
            if len(sys.argv) > 2:
                game = sys.argv[2]
                uninstall(game)
        elif op == 'run':
            if len(sys.argv) > 4:
                game = sys.argv[2]
                if sys.argv[3] == '-i':
                    input_scripts = sys.argv[4]
            if game is not None:
                ENV['GAME_NAME'] = game
            if input_scripts is not None:
                ENV['INPUT_SCRIPTS'] = input_scripts
            inferencing()
        elif op == 'train':
            if len(sys.argv) > 4:
                game = sys.argv[2]
                if sys.argv[3] == '-i':
                    input_scripts = sys.argv[4]
            if game is not None:
                ENV['GAME_NAME'] = game
            if input_scripts is not None:
                ENV['INPUT_SCRIPTS'] = input_scripts
            training()

if __name__ == '__main__':
    main()


import importlib
import json
import os
import sys

def get_module(path: str) -> str:
    # Get the module (the definition of the MLPlay class) name
    if not os.path.isabs(path):
        path = os.path.abspath(path)
    
    sys.path.insert(0, os.path.dirname(path))
    
    module = os.path.basename(path)
    if module.endswith('.py'):
        module = module[:-3]
    
    return module

def import_game(name: str):
    games = {}
    info_path = os.path.join(os.path.dirname(__file__), 'games/games-info.json')
    if os.path.exists(info_path):
        with open(info_path, 'r') as fin:
            games = json.load(fin)
    game_dir = games[name]['path']

    with open(os.path.join(game_dir, 'game.json'), 'r') as fin:
        game_info = json.load(fin)
    game_path = os.path.join(game_dir, game_info['script'])
    module = get_module(game_path)
    game = importlib.import_module(module)
    return game.Game

def import_script(script_path: str):
    module = get_module(script_path)
    ml_play = importlib.import_module(module)
    return ml_play.MLPlay


import os
from stable_baselines3 import PPO

from wrapper import KartWrapper

class MLPlay:
    def __init__(self, env):
        self.env = KartWrapper(env)

        self.model_name = 'kart_model_ppo_vec'
        model_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), self.model_name + '.zip')
        if os.path.exists(model_path):
            self.model = PPO.load(model_path, env=self.env)
        else:
            self.model = PPO('MultiInputPolicy', env=self.env, verbose=1)

    def learn(self, total_timesteps):
        self.model.learn(total_timesteps=total_timesteps, log_interval=1)
        self.model.save(self.model_name)
    
    def reset(self):
        return self.env.reset()
    
    def predict(self, observation):
        action, _states = self.model.predict(observation)
        return action

    def step(self, action):
        return self.env.step(action)


'''
Manage games (where is the execution path, ...) in the PAIAGym.
'''

from io import BytesIO
import json
import os
import sys
import requests
import shutil
from sys import platform
import zipfile
from typing import List

def list_all_games() -> List:
    games_dir = os.path.join(os.path.dirname(__file__), 'games')
    games = [f for f in os.listdir(games_dir) if os.path.isdir(os.path.join(games_dir, f))]
    return games

def list_available_games(mode: str=None) -> List:
    info_path = os.path.join(os.path.dirname(__file__), f'games/games-info.json')
    games = {}
    if os.path.exists(info_path):
        with open(info_path, 'r') as fin:
            games = json.load(fin)
    
    available_games = []
    for name in games:
        if mode is None or games[name]['mode'] == mode:
            available_games.append(name)
    
    return available_games

def download_game(game_path: str) -> None:
    with open(os.path.join(game_path, 'game.json'), 'r') as fin:
        game = json.load(fin)
    
    game_url = None
    if 'url' in game:
        if platform == 'win32':
            if 'windows' in game['url'] and game['url']['windows'] is not None:
                game_url = game['url']['windows']
        elif platform == 'linux' or platform == 'linux2':
            if 'linux' in game['url'] and game['url']['linux'] is not None:
                game_url = game['url']['linux']
        elif platform == 'darwin':
            if 'mac' in game['url'] and game['url']['mac'] is not None:
                game_url = game['url']['mac']
    
    if game_url is not None:
        response = requests.get(game_url)
        zipfile.ZipFile(BytesIO(response.content), 'r').extractall(os.path.join(game_path, 'build'))

def add(name: str, path: str, mode: str='dev') -> None:
    info_path = os.path.join(os.path.dirname(__file__), f'games/games-info.json')
    games = {}
    if os.path.exists(info_path):
        with open(info_path, 'r') as fin:
            games = json.load(fin)
    games[name] = { 'name': name, 'mode': mode, 'path': os.path.abspath(path) }
    with open(info_path, 'w') as fout:
        json.dump(games, fout, indent=4)

def install(name: str) -> None:
    uninstall(name)
    game_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), f'games/{name}')
    download_game(game_path)
    add(name, game_path, mode='prod')

def update(name: str, path: str) -> None:
    remove(name)
    if path is None:
        install(name)
    else:
        add(name, path)

def remove(name: str) -> None:
    info_path = os.path.join(os.path.dirname(__file__), f'games/games-info.json')
    games = {}
    if os.path.exists(info_path):
        with open(info_path, 'r') as fin:
            games = json.load(fin)
    if name in games:
        del games[name]
    with open(info_path, 'w') as fout:
        json.dump(games, fout, indent=4)

def uninstall(name: str) -> None:
    game_path = os.path.join(os.path.dirname(__file__), f'games/{name}/build')
    if os.path.exists(game_path):
        shutil.rmtree(game_path)
    remove(name)

if __name__ == '__main__':
    games = []
    game = None
    path = None
    if len(sys.argv) > 1:
        op = sys.argv[1]
        if op == 'ls':
            games = []
            if len(sys.argv) > 2:
                if sys.argv[2] == '-a':
                    games = list_all_games()
                if sys.argv[2] == '-m':
                    if len(sys.argv) > 3:
                        mode = sys.argv[3]
                        games = list_available_games(mode)
            else:
                games = list_available_games()
            print('\n'.join(games))
        elif op == 'add':
            if len(sys.argv) > 2:
                game = sys.argv[2]
                if len(sys.argv) > 3:
                    path = sys.argv[3]
                add(game, path)
        elif op == 'install':
            if len(sys.argv) > 2:
                game = sys.argv[2]
                install(game)
        elif op == 'update':
            if len(sys.argv) > 2:
                game = sys.argv[2]
                if len(sys.argv) > 3:
                    path = sys.argv[3]
                update(game, path)
        elif op == 'remove':
            if len(sys.argv) > 2:
                game = sys.argv[2]
                remove(game)
        elif op == 'uninstall':
            if len(sys.argv) > 2:
                game = sys.argv[2]
                uninstall(game)